##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################
'''
FormsCollection
$Id: FormsCollection.py,v 1.2 2001/08/09 19:51:51 jack Exp $
'''
__version__='$Revision: 1.2 $'[11:-2]

from string import join

from Acquisition import aq_inner, aq_parent, Explicit
import Globals
from Globals import DTMLFile, PersistentMapping
from AccessControl import ClassSecurityInfo
from OFS.Folder import Folder

from Products.CMFCore import CMFCorePermissions

# portal_forms classes
from Scripts import Scripts
from Templates import Templates
from Forms import Forms
from FormWrapper import FormWrapper


TupleType = type(())

def modifiedOptions():
    # Remove the existing "Properties" option and add our own.
    rval = []
    pos = -1
    for o in Folder.manage_options:
        label = o.get('label', None)
        if label != 'Properties' and label != 'View' and label != 'Contents':
            rval.append(o)
    rval[0:0] = [{'label':'Properties',
                  'action':'manage_propertiesForm'}]
    return tuple(rval)


class FormsCollection(Explicit,Folder):
    meta_type = 'FormsCollection'

    manage_options = (
        {'label': 'Overview', 'action': 'manage_overview'},
        {'label': 'Forms', 'action': 'forms/manage_main'},
        {'label': 'Templates', 'action': 'templates/manage_main'},
        {'label': 'Scripts', 'action': 'scripts/manage_main'},
        ) + modifiedOptions()

    title = ''
    permission_roles = None
    
    forms = None
    templates = None
    selections = None
    scripts = None

    security = ClassSecurityInfo()

    security.declareProtected( CMFCorePermissions.ManagePortal, 'manage_overview' )
    manage_overview = DTMLFile( 'dtml/explainFormsCollection', globals() )

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_propertiesForm')
    manage_propertiesForm = DTMLFile('dtml/FormsCollectionProps', globals())

    def __init__(self, id, title=''):
        self.id = id
	self.title = title
	forms = Forms('forms')
	templates = Templates('templates')
	scripts = Scripts('scripts')
	self._setObject('forms',forms)
	self._setObject('templates',templates)
	self._setObject('scripts',scripts)
        self.selections = PersistentMapping()
	

    def _getSelections(self):
        sels = self.selections
        if sels is None:
            # Backward compatibility.
            self.selections = sels = PersistentMapping()
        return sels

    security.declareProtected(CMFCorePermissions.View, 'render_form')
    def render_form(self,obj,form,view,asobject=1,REQUEST=None):
    	''' render the form in the context of ob '''
	sels = self._getSelections()
	template,scripts = sels.get(form)
	formobj = self.getForm(form)
        ptobj = FormWrapper(self,obj,formobj,template,view,scripts,REQUEST=REQUEST)
	po = ptobj.__of__(obj)
	if asobject:
		return po
	else:
		return po()
	
	
    security.declarePrivate('getForm')
    def getForm(self,id):
        ''' get a form by id '''
	return self.forms._getOb(id)	
	

    security.declarePrivate('getTemplate')
    def getTemplate(self,id):
        ''' get a template by id '''
	return self.templates._getOb(id)	

    security.declarePrivate('getScript')
    def getScript(self,id):
        ''' get a script by id '''
	return self.scripts._getOb(id)	

    security.declarePrivate('formIds')
    def formIds(self):
    	''' return forms ids '''
	return self.forms.objectIds()

    security.declarePrivate('templateIds')
    def templateIds(self):
    	''' return templates ids '''
	return self.templates.objectIds()

    security.declarePrivate('scriptIds')
    def scriptIds(self):
    	''' return scripts ids '''
	return self.scripts.objectIds()

    security.declareProtected(CMFCorePermissions.ManagePortal, 'listViewsForForm')
    def listViewsForForm(self,form):
    	''' list available views for form '''
	sels = self._getSelections()
	template,script = sels.get(form,None)
	if template is not None:
	    tobj = self.templates._getOb(template,None)
	    if tobj is not None:
		return tobj.macros.keys()
	return []
	
    security.declareProtected(CMFCorePermissions.ManagePortal, 'listScriptsForForm')
    def listScriptsForForm(self,form):
    	''' list available scripts for form '''
	sels = self._getSelections()
	template,scripts = sels.get(form,(None,[],))
	return scripts

    security.declareProtected(CMFCorePermissions.ManagePortal, 'listFormsAndViews')
    def listFormsAndViews(self):
    	''' list all forms with template '''
	ret = {}
	for form in self.formIds():
	    ret[form] = self.listViewsForForm(form)
	return ret


    security.declareProtected(CMFCorePermissions.ManagePortal, 'listFormTemplateMapping')
    def listFormTemplateMapping(self):
    	
	forms = self.forms.objectIds()
	sels = self._getSelections()
	ret = []
	
	for form in forms:
	    if sels.has_key(form):
	    	template,scripts = sels[form]
	        ret.append({'form':form,'template':template,'scripts':scripts})
	    else:
	    	ret.append({'form':form,'template':'','scripts':[]})
	return ret

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_properties')
    def manage_properties(self, REQUEST=None):
        '''
        Changes portal_types properties.
        '''
        sels = self._getSelections()

        if REQUEST is not None:
            for key in self.forms.objectIds():
                tname = 'templatemapping_%s' % key
                tval = REQUEST.get(tname,'')
                sname = 'scriptmapping_%s' % key
                svals = REQUEST.get(sname,[])
                sels[key] = (tval,svals,)

	    # strange error with manage_tabs
            # return self.manage_propertiesForm(
	    #     self, REQUEST, manage_tabs_message='Properties changed.')
            return self.manage_propertiesForm(
	        self, REQUEST)


Globals.InitializeClass(FormsCollection)


