##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################

"""Portal forms tool.
$Id: FormsTool.py,v 1.4 2001/08/09 20:04:11 jack Exp $
"""
__version__='$Revision: 1.4 $'[11:-2]


from string import split, join, strip
import Globals
from Globals import DTMLFile, PersistentMapping
from Acquisition import aq_base
from DateTime import DateTime
from AccessControl import ClassSecurityInfo
from OFS.Folder import Folder

from Products.CMFCore.utils import UniqueObject, getToolByName
from Products.CMFCore import CMFCorePermissions

from Products.Formulator.Form import FormValidationError

from FormsContainer import FormsContainer, FormsCollectionPathError
from FormsCollection import FormsCollection


def modifiedOptions():
    # Remove the existing "Properties" option and add our own.
    rval = []
    pos = -1
    for o in Folder.manage_options:
        label = o.get('label', None)
        if label != 'Properties':
            rval.append(o)
    rval[1:1] = [{'label':'Properties',
                  'action':'manage_propertiesForm'}]
    return tuple(rval)


class FormsTool(UniqueObject, FormsContainer, Folder):
    '''
    This tool is used to supply forms to a portal.
    '''

    id = 'portal_forms'
    meta_type = 'CMF Forms Tool'

    security = ClassSecurityInfo()

    manage_options = ( { 'label' : 'Overview', 'action' : 'manage_overview' }
                     , 
                     ) + modifiedOptions()

    def __init__(self):
        self.selections = PersistentMapping()

    def _getSelections(self):
        sels = self.selections
        if sels is None:
            # Backward compatibility.
            self.selections = sels = PersistentMapping()
        return sels

    #
    #   ZMI methods
    #
    security.declareProtected( CMFCorePermissions.ManagePortal, 'manage_overview' )
    manage_overview = DTMLFile( 'dtml/explainFormsTool', globals() )

    request_varname = 'portal_forms'
    selections = None

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_propertiesForm')
    manage_propertiesForm = DTMLFile('dtml/FormsToolProps', globals())



    # limit to FormsContainer
    def all_meta_types(self):
        return (
            {'name': 'FormsCollections',
             'action': 'manage_addFormsCollectionForm',
             'permission': CMFCorePermissions.ManagePortal },)


    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_addFormsCollectionForm')
    manage_addFormsCollectionForm = DTMLFile('dtml/addFormsCollection', globals())

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_addFormsColletion')
    def manage_addFormsCollection(self,id,title='',REQUEST=None):
    	'''
	Adds a Forms collectsion to the Container
	'''
	ob = FormsCollection(id,title)
	self._setObject(id,ob)
	return self.manage_main(self,REQUEST,manage_tabs_message='Folder Added')


    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_properties')
    def manage_properties(self, request_varname='', REQUEST=None):
        '''
        Changes portal_types properties.
        '''
        sels = self._getSelections()

        self.request_varname = str(request_varname)
	
	pt = getToolByName(self,'portal_types')
	typelist = pt.listContentTypes()

        if REQUEST is not None:
            for key in typelist:
                fname = 'typemapping_%s' % key
                val = str(REQUEST[fname])
                if not sels.has_key(key):
		    sels[key] = val
		elif sels[key] != val:
                    self.testFormsCollectionPath(val,1)
                    sels[key] = val
        if REQUEST is not None:
            return self.manage_propertiesForm(
                self, REQUEST, manage_tabs_message='Properties changed.')
    
    
    security.declareProtected(CMFCorePermissions.ManagePortal, 'listFormsCollectionPaths')
    def listFormsCollectionPaths(self):
    	'''
	returns a dict of type: path
	'''
        sels = self._getSelections()
	ret = []
	pt = getToolByName(self,'portal_types')
	typelist = pt.listContentTypes()
	
	for key in typelist:
	    if sels.has_key(key):
	        t={'portaltype':key,'typemapping':sels[key]}
	    else:
	        t={'portaltype':key,'typemapping':''}
	    ret.append(t)
	return ret

    security.declareProtected(CMFCorePermissions.ManagePortal, 'testFormsCollectionPath')
    def testFormsCollectionPath(self,path,raise_exc=0):
    	'''test the given path. raises an error if invalid'''
	
        parts = list(split(path, ','))
        parts.reverse()
        for part in parts:
	    part = strip(part)
            colob = None
            if part == '':
               continue
            if part[:1] == '_':
                # Not allowed.
                colob = None
                if raise_exc:
                    raise FormsCollectionPathError('Underscores are not allowed')
		return 0
            # check for existence
            colob = self.getFormsCollection(part)
            if colob is None:
                # Not found.  Cancel the search.
                if raise_exc:
                    raise FormsCollectionPathError('Name not found: %s' % part)
		return 0
	return 1


    security.declarePublic('listFormsFor')
    def listFormsFor(self,info):
    	'''
	Returns a list of formcontainer_ids that are 
	available for this PortalContent-TypeInfo.
	'''
	sels = self._getSelections()
	path = sels.get(info.Type(),None)
	ret = {}
	if path is not None:
	    collist = self.getFormsCollectionListByPath(path)
	    collist.reverse()
	    for col in collist:
	        forms = col.formIds()
		for form in forms:
		    ret[form] = col.listViewsForForm(form)
	return ret
	
    
    def _getFormCollectionFor(self,ob, formid):
	pt = getToolByName(ob,'portal_types')
	sels = self._getSelections()
	
	ti = pt.getTypeInfo(ob.Type())
	if not ti:
		raise "FormToolsError","Not PortalContent: %s" % ob.meta_type
		
	colobjs = self.getFormsCollectionListByPath(sels.get(ti.Type()))
	colobjs.reverse()
	curcol = None
	for col in colobjs:
	    if formid in col.formIds():
	        curcol = col
	return curcol	    













    #
    #   'portal_forms' interface methods
    #

    security.declareProtected(CMFCorePermissions.View, 'render_form')
    def render_form(self,ob,formid,viewid,REQUEST=None):
    	'''
	this method is usually invoked by a workflow-action
	and runs in context of the content-object.
	the given view of the form is rendered with the values that
	are assigned in the form-definition and the Template that
	is assigned to that form. 	
	'''
	curcol = self._getFormCollectionFor(ob,formid)
	if curcol is not None:
	    if viewid in curcol.listViewsForForm(formid):
	        return curcol.render_form(ob,formid,viewid,REQUEST=REQUEST)
	    else:
	        raise "FormToolsError","No such view %s" % viewid
	else:
	    raise "FormToolsError","No such form %s" % formid
	
	
    security.declareProtected(CMFCorePermissions.View, 'validate_request')
    def validate_request(self,ob,formid,viewid,REQUEST):
    	'''
	this method is called when the form is submitted.
	the REQUEST is passed to the validation of the
	form and the result is validated and returned.
	'''
	curcol = self._getFormCollectionFor(ob,formid)
	if curcol is not None:
	    form = curcol.forms._getOb(formid,None)
	    valid = 0
	    if form is not None:
	    	try:
	            result = form.validate_all_to_request(REQUEST)
		    valid=1
		except FormValidationError,err:
		    result = {}
		    result['errors'] = err.errors
		    result['result'] = err.result
	    
                return (valid,result,)
	else:
	    raise "FormToolsError","No such form %s" % formid

    # should this be ModfyPortalContent ???
    # explicit securitydeclarations are made directly in the scripts
    security.declareProtected(CMFCorePermissions.View, 'render_form')
    def exec_script(self,ob,formid,scriptid,formresult):
        '''
	this method is called when the formresult was validated
	successfully. it it needs to be stored in the same collection
	than the form or accessible via acquisistion.
	'''
	curcol = self._getFormCollectionFor(ob,formid)
	if curcol is not None:
	    script = curcol.scripts._getOb(scriptid,None)
	    if script is not None:
	        so = script.__of__(ob)
                return so(formresult)
	else:
	    raise "FormToolsError","No such form %s" % formid

	

Globals.InitializeClass(FormsTool)
